#Simulation code for Section 5.1

#Note: For comparison with LPFR no GAMMA1TRUE is there
#Note: Fixed term is only intercept, there is no slope for time.
#Note: This program is updated last on Jan 10' 2012 and is working.

set.seed(07171989)

library(nlme)
library(refund)

## SET the global parameters
B =100		

## For generation of W_(t)(s)
kk = (1:100)/100  	
K<- length(kk)
N = 100        	
idvec<- c(1:N)    
mult_X = 0.1     

## For generation of Y_(t)
sd_err<- 0.02  	
sd_int<- 0.05  	
beta0<- 0.06
t1=0		   	
t2=1		   	
t3=2		   	
t4=3           

MSE.Y.PEER1<- c(1:B)
MSE0.PEER1<- c(1:B)
MSE1.PEER1<-  c(1:B)
MSE.Y.D2<- c(1:B)
MSE0.D2<- c(1:B)
MSE1.D2<-  c(1:B)
MSE.Y.IK<-  c(1:B)
MSE0.IK<-  c(1:B)
MSE1.IK<-  c(1:B)

MSE.Y.lpfr<-  c(1:B)
MSE0.lpfr<-  c(1:B)

## Q_mat=col[q1,...,qd]
d=K-1
Q_mat = matrix(0,nrow=K, ncol=d)
for (r in 1:d)
{    Q_mat[,r] = exp(-2500*(kk-r/K)^2)
}
Q_mat1 = matrix(0,nrow=K, ncol=d)
for (r in 1:d)
{    Q_mat1[,r] = exp(-1000*(kk-r/K)^2)
}


#-------------------------------------------------#
#   		1. Penalty	                          #
#-------------------------------------------------# 
#----PEER penalty
Q1<- cbind(Q_mat[,round(0.05*K)], Q_mat1[,round(0.15*K)], 
           exp(-1000*(kk-round(0.30*K)/K)^2), exp(-1000*(kk-round(0.70*K)/K)^2), 
           exp(-1000*(kk-round(0.80*K)/K)^2), exp(-1000*(kk-round(0.90*K)/K)^2),  
           exp(-250*(kk-round(0.50*K)/K)^2), Q_mat[,round(0.95*K)])


P_Q1 <- Q1 %*% solve(t(Q1) %*% Q1) %*% t(Q1)
L1_PEER1<- 10^3*(diag(K)- P_Q1) + 1*P_Q1
L2_PEER1<- 10^3*(diag(K)- P_Q1) + 1*P_Q1

#----D^2 penalty
Left<- cbind(diag(rep(1,K-2)),rep(0,K-2),rep(0,K-2))
Middle<- cbind(rep(0,K-2),diag(rep(-2,K-2)),rep(0,K-2))
Right<- cbind(rep(0,K-2),rep(0,K-2),diag(rep(1,K-2)))
D.2<- rbind(Left+Middle+Right, c(rep(0, K-2), 1, -2), c(rep(0, K-2), 0, 1))

L1_D2<- D.2
L2_D2<- D.2

#----Ridge penalty
L1_IK<- diag(K)  
L2_IK<- diag(K)

#-------------------------------------------------#
#   		2. Simulate Gamma0, Gamma1            #
#-------------------------------------------------# 
## Define Gamma0True (3 bumps)
Gamma0True = 0.2*exp(-500*(kk-round(0.15*K)/K)^2)  - 0.15*exp(-250*(kk-round(0.50*K)/K)^2) + 
             0.15*exp(-500*(kk-round(0.80*K)/K)^2)

Gamma0True = matrix(Gamma0True , nrow=K, ncol=1)

for (bi in 1:B)
{
#-------------------------------------------------#
#   		3. Simulate W	                    #
#-------------------------------------------------# 
GenW<- function(aU=0, bU=0.1)
{
wA=aU + (bU - aU)*matrix(runif(N*1), nrow=N, ncol=1)
mA=aU + (bU - aU)*matrix(runif(N*5), nrow=N, ncol=5) 
nA=aU + (bU - aU)*matrix(runif(N*3), nrow=N, ncol=3) 

WideBump <- NarrowBump <- MedBump <- matrix(0,nrow=N,ncol=K)

for (i in 1:N)
{
NarrowBump[i,] = (nA[i,1]+.1)*Q_mat[,round(0.05*K)]+ (nA[i,2]+.1)*Q_mat[,round(0.95*K)] # sharp bump

MedBump[i,] = (mA[i,1]+0.6)*exp(-1000*(kk-round(0.15*K)/K)^2) +
               (mA[i,1]+0.6)*exp(-1000*(kk-round(0.30*K)/K)^2) +
              (mA[i,2]+0.5)*exp(-1000*(kk-round(0.70*K)/K)^2) +     
              (mA[i,3]+0.5)*exp(-1000*(kk-round(0.80*K)/K)^2) + 
                (mA[i,4]+0.4)*exp(-1000*(kk-round(0.90*K)/K)^2) # medium gaussian   
                                           
WideBump[i,] = (wA[i,1]+0.5)*exp(-250*(kk-round(0.50*K)/K)^2) # big gaussian 
}


Xtrue = WideBump + MedBump + NarrowBump

varX = mean(apply(Xtrue, 1, var))   #S_i^2
noise_X = mult_X*sqrt(varX)         #Noise  in X
X = Xtrue + noise_X * matrix(rnorm(length(Xtrue)), nrow=N, ncol=K)

return(X)
}

W1<- GenW(0, 0.1)
W1<- matrix(unlist(W1), nrow=N, ncol=K)

W2<- GenW(0, 0.1)
W2<- matrix(unlist(W2), nrow=N, ncol=K)

W3<- GenW(0, 0.1)
W3<- matrix(unlist(W3), nrow=N, ncol=K)

W4<- GenW(0, 0.1)
W4<- matrix(unlist(W4), nrow=N, ncol=K)

t<- c(rep(t1,nrow(W1)), rep(t2,nrow(W2)), rep(t3,nrow(W3)), rep(t4,nrow(W4)))
Wcol1<- rbind(W1, W2, W3, W4)
W<- cbind(Wcol1)
W1_PEER1<- Wcol1 %*% solve(L1_PEER1)
W1_D2<- Wcol1 %*% solve(L1_D2)


#-------------------------------------------------#
#   		4. Simulate Y                         #
#-------------------------------------------------#         
## f1(t)=t
Y1<- matrix(0, nrow=N, ncol=1)
Y2<- matrix(0, nrow=N, ncol=1)
Y3<- matrix(0, nrow=N, ncol=1)
Y4<- matrix(0, nrow=N, ncol=1)
for (i in 1:N)
	{
	tx<- rnorm(1)
	Y1[i,1] =beta0+ W1[i,]%*% Gamma0True + sd_int*tx + sd_err*rnorm(1)
	Y2[i,1] =beta0+ W2[i,]%*% Gamma0True + sd_int*tx + sd_err*rnorm(1)
	Y3[i,1] =beta0+ W3[i,]%*% Gamma0True + sd_int*tx + sd_err*rnorm(1)
	Y4[i,1] =beta0+ W4[i,]%*% Gamma0True + sd_int*tx + sd_err*rnorm(1)
	}

Y<-  rbind(Y1, Y2, Y3, Y4)

#-------------------------------------------------#
#   		5. Fitting model                      #
#-------------------------------------------------#  
id.bd1<- factor(rep(1, nrow(Y)))
t<- c(rep(t1,N), rep(t2,N), rep(t3,N), rep(t4,N))

library(nlme)
idmat<- rbind(diag(N),diag(N), diag(N), diag(N))

#---5.1 Fitting PEER1
out_PEER1<- lme(fixed=Y~1,
          	   random=list(id.bd1=pdBlocked(list(pdIdent(~W1_PEER1-1), 
	    		    				       pdIdent(~idmat-1)
                                                 )
                                            )
                           )
                ) 

Gamma_PEER1<-matrix(out_PEER1$coeff$random$id.bd1, ncol=1)
r1<- K+1
r2<- 2*K 
Gamma0.PEER1.hat<- matrix(Gamma_PEER1[1:K], ncol=1)

beta.hat.PEER1<- matrix(out_PEER1$coeff$fixed, ncol=1)
X<- cbind(c(rep(1, nrow(Y))))                 #Need to be changed problem to problem
Y.hat.PEER1<- X%*%beta.hat.PEER1+W1_PEER1%*%Gamma0.PEER1.hat
Gamma0.hat.PEER1<- solve(L1_PEER1)%*%Gamma0.PEER1.hat


#---5.2 Fitting D2
out_D2<- lme(fixed=Y~1,
          	   random=list(id.bd1=pdBlocked(list(pdIdent(~W1_D2-1), 
	    		    				       pdIdent(~idmat-1)
                                                 )
                                            )
                           )
                ) 

Gamma_D2<-matrix(out_D2$coeff$random$id.bd1, ncol=1)
r1<- K+1
r2<- 2*K 
Gamma0.D2.hat<- matrix(Gamma_D2[1:K], ncol=1)

beta.hat.D2<- matrix(out_D2$coeff$fixed, ncol=1)
X<- cbind(c(rep(1, nrow(Y))))                 #Need to be changed problem to problem
Y.hat.D2<- X%*%beta.hat.D2+W1_D2%*%Gamma0.D2.hat
Gamma0.hat.D2<- solve(L1_D2)%*%Gamma0.D2.hat

#---5.3 Fitting IK
out_IK<- lme(fixed=Y~1,
          	 random=list(id.bd1=pdBlocked(list(pdIdent(~Wcol1-1), 
	    		    				     pdIdent(~idmat-1)
                                               )
                                         )
                          )		    				 
             ) 


Gamma_IK<-matrix(out_IK$coeff$random$id.bd1, ncol=1)
r1<- K+1
r2<- 2*K 
Gamma0.hat.IK<- matrix(Gamma_IK[1:K], ncol=1)

beta.hat.IK<- matrix(out_IK$coeff$fixed, ncol=1)
X<- cbind(c(rep(1, nrow(Y))))                 #Need to be changed problem to problem
Y.hat.IK<- X%*%beta.hat.IK+Wcol1%*%Gamma0.hat.IK

#---5.4 Fitting LPFR
out_lpfr = lpfr(Y=Y, subj=id.bd1, funcs = Wcol1, smooth.cov=FALSE)
Gamma0.hat.lpfr<- out_lpfr$BetaHat[[1]]
beta.hat.lpfr<- out_lpfr$beta.covariates
Y.hat.lpfr<- out_lpfr$fitted.vals
out.madan<<- out_lpfr

#-------------------------------------------------#
#   		6. MSE                      		  #
#-------------------------------------------------#  
MSE.Y.PEER1[bi]<- unlist(t(Y.hat.PEER1 - Y)%*%(Y.hat.PEER1 - Y))
MSE0.PEER1[bi]<- unlist(t(Gamma0.hat.PEER1 - Gamma0True)%*%(Gamma0.hat.PEER1 - Gamma0True))

MSE.Y.D2[bi]<- unlist(t(Y.hat.D2 - Y)%*%(Y.hat.D2 - Y))
MSE0.D2[bi]<- unlist(t(Gamma0.hat.D2 - Gamma0True)%*%(Gamma0.hat.D2 - Gamma0True))

MSE.Y.IK[bi]<- unlist(t(Y.hat.IK - Y)%*%(Y.hat.IK - Y))
MSE0.IK[bi]<- unlist(t(Gamma0.hat.IK - Gamma0True)%*%(Gamma0.hat.IK - Gamma0True))

MSE.Y.lpfr[bi]<- unlist(t(Y.hat.lpfr - Y)%*%(Y.hat.lpfr - Y))
MSE0.lpfr[bi]<- unlist(t(Gamma0.hat.lpfr - Gamma0True)%*%(Gamma0.hat.lpfr - Gamma0True))

if(bi==1) 
	{
	Gamma0.hat.PEER1.cum<- Gamma0.hat.PEER1
	Gamma0.hat.D2.cum<- Gamma0.hat.D2
	Gamma0.hat.IK.cum<- Gamma0.hat.IK
	Gamma0.hat.lpfr.cum<- Gamma0.hat.lpfr
	}

if(bi>1) 
	{
	Gamma0.hat.PEER1.cum<- cbind(Gamma0.hat.PEER1.cum, Gamma0.hat.PEER1)
	Gamma0.hat.D2.cum<- cbind(Gamma0.hat.D2.cum, Gamma0.hat.D2)
	Gamma0.hat.IK.cum<- cbind(Gamma0.hat.IK.cum, Gamma0.hat.IK)
	Gamma0.hat.lpfr.cum<- cbind(Gamma0.hat.lpfr.cum, Gamma0.hat.lpfr)
	}

print(bi)
}

Gamma0.hat.PEER1.m<- apply(Gamma0.hat.PEER1.cum,1 , mean)
Gamma0.hat.D2.m<- apply(Gamma0.hat.D2.cum,1 , mean)
Gamma0.hat.IK.m<- apply(Gamma0.hat.IK.cum,1 , mean)
Gamma0.hat.lpfr.m<- apply(Gamma0.hat.lpfr.cum,1 , mean)

#-------------------------------------------------#
#   		Summary of  MSE              		  #
#-------------------------------------------------#
MSE.Y<- cbind(MSE.Y.PEER1, MSE.Y.D2, MSE.Y.IK, MSE.Y.lpfr)
MSE0<- cbind(MSE0.PEER1, MSE0.D2, MSE0.IK, MSE0.lpfr)

MSE.Y
MSE0

mean(MSE.Y.PEER1)
mean(MSE.Y.D2)
mean(MSE.Y.IK)
mean(MSE.Y.lpfr)

mean(MSE0.PEER1)
mean(MSE0.D2)
mean(MSE0.IK)
mean(MSE0.lpfr)

save(MSE.Y, MSE0, Gamma0.hat.PEER1.cum, Gamma0.hat.D2.cum,
                  Gamma0.hat.IK.cum, Gamma0.hat.lpfr.cum, 
     file='lpfr.rda')
