rm(list=ls())

#Note: For comparison with PEER, Ridge and D2. No comparison with LPFR
#Note: Fixed term includes time slope, that's why LPFR is not applicable.
#Note: This program is updated last on Jan 10' 2012 and is working.

library(nlme)

##SET path
#plot.path<- "I:/Projects/MRS_Imaging/LCmodel/Processed/Madan/Final/For LMMP EER Simulation/Simulation/"

## SET the global parameters
B =100		# Number of simulation

## For generation of X_(t)(s)
kk = (1:100)/100  	# Data on (0,1]
K<- length(kk)
N = 200        	# Sample size (Number of X_i(t) 
A<- seq(0.5, 3, 0.25)		# Penalty index
idvec<- c(1:N)    # List of id's
mult_X = 0.1     	# Multiplier of noise in X (Reciprocal of Signal to Noise ratio)

## For generation of Y_(t)
R2<- 0.60  		# R^2
sd_int<- 0.05  	# sd of random intercept
beta0<- 0.06
beta1<- 0.02
t1=0		   	# Time1
t2=0.25		   	# Time2
t3=0.5		   	# Time3
t4=1           	# Time4

## Q_mat=col[q1,...,qd]
d=K-1
Q_mat = matrix(0,nrow=K, ncol=d)
for (r in 1:d)
{    Q_mat[,r] = exp(-2500*(kk-r/K)^2)
}
Q_mat1 = matrix(0,nrow=K, ncol=d)
for (r in 1:d)
{    Q_mat1[,r] = exp(-1000*(kk-r/K)^2)
}

#-------------------------------------------------#
#   		1. Penalty	                          #
#-------------------------------------------------# 
#----PEER penalty
Q1<- cbind(Q_mat[,round(0.05*K)], Q_mat1[,round(0.15*K)], 
           exp(-1000*(kk-round(0.30*K)/K)^2), exp(-1000*(kk-round(0.70*K)/K)^2), 
           exp(-1000*(kk-round(0.80*K)/K)^2), exp(-1000*(kk-round(0.90*K)/K)^2),  
           exp(-250*(kk-round(0.50*K)/K)^2), Q_mat[,round(0.95*K)])

P_Q1 <- Q1 %*% solve(t(Q1) %*% Q1) %*% t(Q1)

#-------------------------------------------------#
#   		2. Simulate Gamma0, Gamma1            #
#-------------------------------------------------# 
## Define Gamma0True & Gamma1True
Gamma0True = 0.2*exp(-500*(kk-round(0.15*K)/K)^2)  - 0.15*exp(-250*(kk-round(0.50*K)/K)^2) + 
             0.15*exp(-500*(kk-round(0.80*K)/K)^2)

Gamma1True = 0.2*exp(-500*(kk-round(0.30*K)/K)^2)- 0.2*Q_mat1[,round(0.70*K)]

Gamma0True = matrix(Gamma0True , nrow=K, ncol=1)
Gamma1True = matrix(Gamma1True , nrow=K, ncol=1)
#-------------------------------------------------#
#   		3. Simulate W	                    #
#-------------------------------------------------# 
GenW<- function(aU=0, bU=0.1)
{
wA=aU + (bU - aU)*matrix(runif(N*1), nrow=N, ncol=1)
mA=aU + (bU - aU)*matrix(runif(N*5), nrow=N, ncol=5) 
nA=aU + (bU - aU)*matrix(runif(N*3), nrow=N, ncol=3) 

WideBump <- NarrowBump <- MedBump <- matrix(0,nrow=N,ncol=K)

for (i in 1:N)
{
NarrowBump[i,] = (nA[i,1]+.1)*Q_mat[,round(0.05*K)]+ (nA[i,2]+.1)*Q_mat[,round(0.95*K)]

MedBump[i,] = (mA[i,1]+0.6)*exp(-1000*(kk-round(0.15*K)/K)^2) +
               (mA[i,1]+0.6)*exp(-1000*(kk-round(0.30*K)/K)^2) +
              (mA[i,2]+0.5)*exp(-1000*(kk-round(0.70*K)/K)^2) +     
              (mA[i,3]+0.5)*exp(-1000*(kk-round(0.80*K)/K)^2) + 
                (mA[i,4]+0.4)*exp(-1000*(kk-round(0.90*K)/K)^2) # medium gaussian   
                                           
WideBump[i,] = (wA[i,1]+0.5)*exp(-250*(kk-round(0.50*K)/K)^2) # big gaussian 
}


Xtrue = WideBump + MedBump + NarrowBump

varX = mean(apply(Xtrue, 1, var))   #S_i^2
noise_X = mult_X*sqrt(varX)         #Noise  in X
X = Xtrue + noise_X * matrix(rnorm(length(Xtrue)), nrow=N, ncol=K)

return(X)
}







#------Initializing R objects

Gamma0.hat.cum<- Gamma1.hat.cum<- NULL
MSE.Y <- MSE0 <- MSE1<- AIC<- LogLik <- matrix(0, nrow=B, ncol=length(A))

#------Start of simulation
bi<- 1

while(bi<=B)
{
print(bi)

W1<- GenW(0, 0.1)
W1<- matrix(unlist(W1), nrow=N, ncol=K)
#matplot(t(W1), type='l')

W2<- GenW(0, 0.1)
W2<- matrix(unlist(W2), nrow=N, ncol=K)
#matplot(t(W2), type='l')

W3<- GenW(0, 0.1)
W3<- matrix(unlist(W3), nrow=N, ncol=K)
#matplot(t(W3), type='l')

W4<- GenW(0, 0.1)
W4<- matrix(unlist(W4), nrow=N, ncol=K)
#matplot(t(W4), type='l')

t<- c(rep(t1,nrow(W1)), rep(t2,nrow(W2)), rep(t3,nrow(W3)), rep(t4,nrow(W4)))
Wcol1<- rbind(W1, W2, W3, W4)
Wcol2<- rbind(t1*W1, t2*W2, t3*W3, t4*W4)
W<- cbind(Wcol1)

#-------------------------------------------------#
#   		4. Simulate Y                         #
#-------------------------------------------------#         
## f1(t)=t
Y1<- matrix(0, nrow=N, ncol=1)
Y2<- matrix(0, nrow=N, ncol=1)
Y3<- matrix(0, nrow=N, ncol=1)
Y4<- matrix(0, nrow=N, ncol=1)
for (i in 1:N)
	{
	tx<- rnorm(1)
	Y1[i,1] =beta0+ beta1*t1+ W1[i,]%*% Gamma0True + t1*W1[i,]%*%Gamma1True 
	Y2[i,1] =beta0+ beta1*t2+ W2[i,]%*% Gamma0True + t2*W2[i,]%*%Gamma1True 
	Y3[i,1] =beta0+ beta1*t3+ W3[i,]%*% Gamma0True + t3*W3[i,]%*%Gamma1True 
	Y4[i,1] =beta0+ beta1*t4+ W4[i,]%*% Gamma0True + t4*W4[i,]%*%Gamma1True 
	}

Y<- cbind(Y1, Y2, Y3, Y4)
varY = mean(apply(Y, 2, var))  # Choose the 'err_Y' below based on the std(Y)

## Add noise to Y. 
# Follow the P. Reiss paper with R^2 = 0.6 or 0.8 - defined in GLOBAL Parms

sd_err = sqrt(varY*(1-R2)/R2)

Y1<- matrix(0, nrow=N, ncol=1)
Y2<- matrix(0, nrow=N, ncol=1)
Y3<- matrix(0, nrow=N, ncol=1)
Y4<- matrix(0, nrow=N, ncol=1)
tx<- rnorm(1)
for (i in 1:N)
	{
	tx<- rnorm(1)
	Y1[i,1] =beta0+ beta1*t1+ W1[i,]%*% Gamma0True + t1*W1[i,]%*%Gamma1True + sd_int*tx + sd_err*rnorm(1)
	Y2[i,1] =beta0+ beta1*t2+ W2[i,]%*% Gamma0True + t2*W2[i,]%*%Gamma1True + sd_int*tx + sd_err*rnorm(1)
	Y3[i,1] =beta0+ beta1*t3+ W3[i,]%*% Gamma0True + t3*W3[i,]%*%Gamma1True + sd_int*tx + sd_err*rnorm(1)
	Y4[i,1] =beta0+ beta1*t4+ W4[i,]%*% Gamma0True + t4*W4[i,]%*%Gamma1True + sd_int*tx + sd_err*rnorm(1)
	}

Y<-  rbind(Y1, Y2, Y3, Y4)

id.bd1<- factor(rep(1, nrow(Y)))
t<- c(rep(t1,N), rep(t2,N), rep(t3,N), rep(t4,N))

idmat<- rbind(diag(N),diag(N), diag(N), diag(N))

#-------------------------------------------------#
#   		5. Fitting model                      #
#-------------------------------------------------#  

#----PEER penalty
PEERfn<- function(a)
	{
	L1_PEER1<- 10^a*(diag(K)- P_Q1) + 1*P_Q1
	L2_PEER1<- 10^a*(diag(K)- P_Q1) + 1*P_Q1
	W1_PEER1<- Wcol1 %*% solve(L1_PEER1)
	W2_PEER1<- Wcol2 %*% solve(L2_PEER1)

	#---5.1.1 Fitting PEER1
	success.fit<- 0 #Indicator whether fitting is success(=1) or not(=0)
	try(out_PEER1<- lme(fixed=Y~t,
          	   random=list(id.bd1=pdBlocked(list(pdIdent(~W1_PEER1-1), 
								 pdIdent(~W2_PEER1-1),
	    		    				       pdIdent(~idmat-1)
                                                 )
                                            )
                           )
                ) , silent=T)

	if(exists('out_PEER1')){
	success.fit<- 1 #Indicator whether fitting is success)=1) or not(=0)

	Gamma_PEER1<-matrix(out_PEER1$coeff$random$id.bd1, ncol=1)
	r1<- K+1
	r2<- 2*K 
	Gamma0.PEER1.hat<- matrix(Gamma_PEER1[1:K], ncol=1)
	Gamma1.PEER1.hat<- matrix(Gamma_PEER1[r1:r2], ncol=1)

	beta.hat.PEER1<- matrix(out_PEER1$coeff$fixed, ncol=1)
	X<- cbind(c(rep(1, nrow(Y))),t)                 #Need to be changed problem to problem
	Y.hat.PEER<- X%*%beta.hat.PEER1+W1_PEER1%*%Gamma0.PEER1.hat+W2_PEER1%*%Gamma1.PEER1.hat
	Gamma0.hat<- solve(L1_PEER1)%*%Gamma0.PEER1.hat
	Gamma1.hat<- solve(L2_PEER1)%*%Gamma1.PEER1.hat

	#---------MSE & AIC
	MSE.Y<- unlist(t(Y.hat.PEER - Y)%*%(Y.hat.PEER - Y))
	MSE0<- unlist(t(Gamma0.hat - Gamma0True)%*%(Gamma0.hat - Gamma0True))
	MSE1<- unlist(t(Gamma1.hat - Gamma1True)%*%(Gamma1.hat - Gamma1True))

	AIC<- AIC(out_PEER1)
	LogLik<-  out_PEER1$logLik

	return(list(AIC=AIC, LogLik=LogLik, MSE.Y=MSE.Y, success.fit=success.fit,
            MSE0=MSE0, MSE1=MSE1, 
            Gamma0.hat=Gamma0.hat, Gamma1.hat=Gamma1.hat))
			}

	if(success.fit==0) return(list(success.fit=success.fit))
	
	}

#Running for each 'a'
	Gamma0.hat<- Gamma1.hat<- NULL	
for (ai in 1:length(A))
	{
	success=1
	rm(out)
	a<- A[ai]
	print(a)

	out<- PEERfn(a)

	if(out$success.fit==0){
		success<- 0
		Gamma0.hat<- Gamma1.hat<- NULL
		break
	}

	Gamma0.hat<- cbind(Gamma0.hat, out$Gamma0.hat)
	Gamma1.hat<- cbind(Gamma1.hat, out$Gamma1.hat)
	AIC[bi, ai]<- out$AIC
	LogLik[bi, ai]<- out$LogLik
	MSE.Y[bi, ai]<- out$MSE.Y
	MSE0[bi, ai]<- out$MSE0
	MSE1[bi, ai]<- out$MSE1
	}


Gamma0.hat.cum<- cbind(Gamma0.hat.cum, Gamma0.hat)
Gamma1.hat.cum<- cbind(Gamma1.hat.cum, Gamma1.hat)

print(bi)

bi<- bi+success
}

filename=paste('Gamma.N', N, '.Rsq', R2,'.sdint', sd_int, '.rda', sep='')
save(Gamma0.hat.cum, Gamma1.hat.cum, AIC, LogLik, MSE.Y, MSE0, MSE1,
     file=filename)


