#library(refund)
#----------------------------------------------------------------#
#   1. IMPORTING DATA							     #
#----------------------------------------------------------------# 
#rm(list=ls())
#setwd("I:/Projects/MRS_Imaging/LCmodel/Processed/Madan/Final/LMM PEER/Analysis")
K<- 399    #Number of grid points
kk = (1:K)/K

#Importing data
bg<- read.csv('BASALGANGLIA_GDS.csv', header=T)

#----------------------------------------------------------------#
#   2. PLOTTING OF W'S							     #
#----------------------------------------------------------------# 
names(bg)
dim(bg)

#t=0
bg0<- subset(bg, t==0)
bg1<- bg0[,7:405]
bg2<- t(bg1)

#t>0
bg0.1<- subset(bg, t>0)
bg1.1<- bg0.1[,7:405]
bg2.1<- t(bg1.1)

#pdf(file="BG_Explore.pdf")
matplot(bg2, type='l', main='Basal Ganglia (t=0)')
matplot(bg2.1, type='l', main='Basal Ganglia (t>0)')
#dev.off()

table(tapply(bg$t, bg$ID, max))


#-------------------------------------------------#
#   3. Q, L_Q, W and ID                 		  #
#-------------------------------------------------# 
#Importing Q matrix
rdafiles<- dir(pattern=".rda")
load("Q_HARBOR_LFWM.rda")
dim(Q)

Y<- bg$GDS
t<- bg$t/365.25
id<- bg$ID
W<- bg[,7:405]
W<- as.matrix(W)

#Normalizing W matrix
norm<- sqrt(diag(W%*%t(W)))
W<- W/norm


Male<- ifelse(bg$sex=='M', 1,0)
t2<- t^2

#Fit0<- lpeer(Y=Y, subj=id, t=t, covariates=t, funcs=W, 
#	    pentype='DECOMP', Q=Q, se=T, a=10)
#Fit0$AIC #-395.2046
#plot.lpeer(Fit0)

#-----------------------------------------------------
Fit1<- lpeer(Y=Y, subj=id, t=t, covariates=t, funcs=W, 
	    pentype='DECOMP', f_t=cbind(1,t), Q=Q, se=T, a=10)
Fit1$AIC #395.2335
#summary(Fit1$fit)
#plot.lpeer(Fit1)

#-----------------------------------------------------
Fit2<- lpeer(Y=Y, subj=id, t=t, covariates=cbind(bg$age0,t), funcs=W, 
	    pentype='DECOMP', f_t=cbind(1,t), Q=Q, se=T, a=10)
Fit2$AIC #2.732171

Fit3<- lpeer(Y=Y, subj=id, t=t, covariates=cbind(Male,t), funcs=W, 
	    pentype='DECOMP', f_t=cbind(1,t), Q=Q, se=T, a=10)
Fit3$AIC #2.738232

Fit4<- lpeer(Y=Y, subj=id, t=t, covariates=cbind(bg$race,t), funcs=W, 
	    pentype='DECOMP', f_t=cbind(1,t), Q=Q, se=T, a=10)
Fit4$AIC #2.72859

Fit5<- lpeer(Y=Y, subj=id, t=t, covariates=cbind(t, t2), funcs=W, 
	    pentype='DECOMP', f_t=cbind(1,t, t2), Q=Q, se=T, a=10)
Fit5$AIC #2.718135
#plot.lpeer(Fit5)

Fit6<- lpeer(Y=Y, subj=id, t=t, covariates=t, funcs=W, 
	    pentype='DECOMP', f_t=cbind(1,t), Q=Q, se=T, a=100)
Fit6$AIC #2.772558


















Fit7<- lpeer(Y=Y, subj=id, t=t, covariates=t, funcs=W, 
	    pentype='DECOMP', f_t=cbind(1,t), Q=Q, se=T, a=sqrt(10))
Fit7$AIC #394.9788
plot.lpeer(Fit7)

logt<- log(1+t)
Fit8<- lpeer(Y=Y, subj=id, t=t, covariates=cbind(t, logt), funcs=W, 
	    pentype='DECOMP', f_t=cbind(1,logt), Q=Q, se=T, a=10)
Fit8$AIC #2.779175
#plot.lpeer(Fit8)

expt<- exp(t)-1
Fit9<- lpeer(Y=Y, subj=id, t=t, covariates=cbind(t, expt), funcs=W, 
	    pentype='DECOMP', f_t=cbind(1,expt), Q=Q, se=T, a=10)
Fit9$AIC #2.645854
#plot.lpeer(Fit9)

#-------------------------------------------------------------
#----   Summary of Fit6         -----------------------------#
#-------------------------------------------------------------
Gamma0.hat.PEER<- Fit6$GammaHat[,1]
Gamma1.hat.PEER<- Fit6$GammaHat[,2]
SE.gamma0<- Fit6$se.Gamma[,1]
SE.gamma1<- Fit6$se.Gamma[,2]
names(Fit6)
Fit6$fit
Fit6$N
Fit6$TotalObs
Fit6$sigma^2
Fit6$Sigma.u^2

#----------------------------------------------------------------#
#                          PLOTTING                              #
#----------------------------------------------------------------# 
pdf(file = "Application1.pdf", wid=9, hei=4.50)
par(mfrow=c(1,2))
matplot(kk, bg2, type='l', xlab='s', ylab='', main='Observed MR spectra')
range0.1=range(Q)
plot(kk,t(Q)[,1], ylim=range0.1, lty=1, type='l',  xlab='s', ylab='', 
     main='Pure metabolite spectra', col=1)
points(kk, t(Q)[,2], type='l', lty=1, col=2)
points(kk, t(Q)[,3], type='l', lty=1, col=3)
points(kk, t(Q)[,4], type='l', lty=1, col=4)
points(kk, t(Q)[,5], type='l', lty=1, col=5)
points(kk, t(Q)[,6], type='l', lty=1, col=6)
points(kk, t(Q)[,7], type='l', lty=1, col=7)
points(kk, t(Q)[,8], type='l', lty=1, col=8)
points(kk, t(Q)[,9], type='l', lty=1, col=9)
legend(x=300/K, y=range0.1[2], col=c(1:9), lty=rep(1,9), cex=0.50,
       legend=c('Cr', 'Glu', 'Glc', 'GPC', 'Ins', 'NAA', 'NAAG', 'Scyllo', 'Tau'))

dev.off()

pdf(file = "Application2.pdf", wid=9, hei=7)
par(mfrow=c(1,1))
yrange=range(Q, Gamma0.hat.PEER, Gamma0.hat.PEER-1.96*SE.gamma0, 
             Gamma0.hat.PEER+1.96*SE.gamma0) + c(-0.01, 0.5)
#yrange=c(max(abs(yrange)), -max(abs(yrange)))
plot(kk,Gamma0.hat.PEER, type='l', ylab='', ylim=yrange,
     main=expression(tilde(gamma[0])), xlab='s', lwd=2, cex.main=2)
polygon(c(kk,rev(kk)),c(Gamma0.hat.PEER+1.96*SE.gamma0,rev(Gamma0.hat.PEER-1.96*SE.gamma0)),col="grey",border=NA) 
points(kk,Gamma0.hat.PEER, type='l', lwd=2)
points(kk, t(Q)[,1]+0.5, type='l', lty=1, col=2)
points(kk, t(Q)[,2]+0.5, type='l', lty=1, col=3)
points(kk, t(Q)[,6]+0.5, type='l', lty=1, col=4)
points(kk, t(Q)[,7]+0.5, type='l', lty=1, col=5)
points(kk, t(Q)[,8]+0.5, type='l', lty=1, col=6)
abline(h=0)
legend('topright', col=c(1, 2:6), lty=c(1,rep(1,5)), cex=1.2, lwd=c(2,rep(1,5)),
       legend=c(expression(tilde(gamma[0])), 'Cr', 'Glu', 'NAA', 'NAAG', 'Scyllo'))
dev.off()

pdf(file = "Application3.pdf", wid=9, hei=7)
par(mfrow=c(1,1))
yrange=range(Gamma1.hat.PEER, Gamma1.hat.PEER-1.96*SE.gamma1, 
             Gamma1.hat.PEER+1.96*SE.gamma1) + 2*c(-0.01, 0.5)/10e2
yrange1=c(yrange[1], yrange[2]+0.2)
plot(kk,Gamma1.hat.PEER, type='l', ylim=yrange1, ylab='', 
     main=expression(tilde(gamma[1])), xlab='s', lwd=2, cex.main=2)
polygon(c(kk,rev(kk)),c(Gamma1.hat.PEER+1.96*SE.gamma1,rev(Gamma1.hat.PEER-1.96*SE.gamma1)),col="grey",border=NA) 
points(kk,Gamma1.hat.PEER, type='l', lwd=2)
points(kk, t(Q)[,1]/3+0.15, type='l', lty=1, col=2)
points(kk, t(Q)[,2]/3+0.15, type='l', lty=1, col=3)
points(kk, t(Q)[,6]/3+0.15, type='l', lty=1, col=4)
points(kk, t(Q)[,7]/3+0.15, type='l', lty=1, col=5)
points(kk, t(Q)[,8]/3+0.15, type='l', lty=1, col=6)
abline(h=0)
legend('topright', col=c(1, 2:6), lty=c(1,rep(1,5)), cex=1.2, lwd=c(2,rep(1,5)),
       legend=c(expression(tilde(gamma[1])), 'Cr', 'Glu', 'NAA', 'NAAG', 'Scyllo'))
dev.off()

pdf(file = "Application4.pdf", wid=9, hei=4.50)
par(mfrow=c(1,2), mar=c(5.1, 5.1, 4.1, 2.1))
plot(Y, Fit6$fitted.vals[,2], ylab=expression(tilde(y)), xlim=c(0,4), ylim=c(0,4),
     main='Prediction performance', xlab='y', cex.main=1, cex=0.5)
abline(0,1, lty=2)
plot(Fit6$fitted.vals[,2], Fit6$fitted.vals[,2]-Y, xlab=expression(tilde(y)), xlim=c(0,4), ylim=c(-1.5,1.5),
     main='Residual plot', ylab=expression(tilde(y)-y), cex.main=1, cex=0.5)
abline(h=0,  lty=2)
dev.off()

#Fit.Ridge<- lpeer(Y=Y, subj=id, t=t, covariates=cbind(t), funcs=W, 
#	    f_t=cbind(1,t), se=F)

#plot.lpeer(Fit.Ridge)

simple.lme<- lme(fixed=Y~t, random=~1|id) 
logLik(simple.lme)# -192.7365 (df=4)
AIC(simple.lme) # 393.4729

simple.lme2<- lme(fixed=Y~t, random=~t|id) 
logLik(simple.lme2)#  -183.2717 (df=6)
AIC(simple.lme2) # 378.5435

age0<- bg$age0
race<- bg$race
simple.lme_cov<- lme(fixed=Y~t+age0+Male+race, random=~1|id) 
logLik(simple.lme_cov)# -196.3253 (df=7)
AIC(simple.lme_cov) # 406.6505

simple.lme2_cov<- lme(fixed=Y~t+age0+Male+race, random=~t|id) 
logLik(simple.lme2_cov)# -187.6983 (df=9) 
AIC(simple.lme2_cov) # 393.3967
summary(simple.lme2_cov)
VarCorr(simple.lme2_cov, rdig=4)[,2]

logLik(Fit6$fit) # -191.6835 (df=6)
AIC(Fit6$fit) # 395.367

#Summary of findings;
#--------------------
#simple.lme_cov is better simple.lme2_cov: 


